-- Prep module
local M = {}
local ugfxCalled = 0

-- For doing stuff
local timeMax = 0.75
local timeElap = 0

----------------
-- Small Grid --
----------------
--local carrierX = 0
--local carrierY = 0
--local carrierZ = 30
--local carrierPhi = 180

-----------
-- Italy --
-----------
--local carrierX = 90.970
--local carrierY = 2141.157
--local carrierZ = 142.600
--local carrierPhi = 88.344

--local carrierX = -628.332
--local carrierY = -1615.750
--local carrierZ = 142.600
--local carrierPhi = 172.725

------------------------
-- Jungle Rock Island --
------------------------
--local carrierX = 209.929
--local carrierY = 1070.681
--local carrierZ = 129.140
--local carrierPhi = -83.010

--------------------
-- Sandy Mountain --
--------------------
--local carrierX = 38.000
--local carrierY = -611.000
--local carrierZ = 924.600
--local carrierPhi = 16.000

--local carrierX = 209.929
--local carrierY = 1070.681
--local carrierZ = 129.140
--local carrierPhi = -83.010

-- Light stuff
local lightBrightness = 1.0
local lightFlareScale = 0.5
local lightFlareType = "vehicleDefaultLightflare"
local lightMaxCount = 38
local lightRadius = 20
local lightOffsetsColors = {

		-- Start of runway
		{-27.064,	-207.409,	10.2,	1,	1,	1},
		{-17.1175,	-205.682,	10.2,	1,	1,	1},
		{-7.171,	-203.955,	10.2,	1,	1,	1},
		{-12.14425,	-204.8185,	10.2,	1,	1,	1},
		{-22.09075, -206.5455,	10.2,	1,	1,	1},

		-- End of runway
		{-43.987,	-44.489,	10.2,	1,	1,	1},
		{-25.733,	-24.342,	10.2,	1,	1,	1},

		-- Starboard runway line
		{-24.262,	-40.981,	10.2,	1,	1,	1},
		{-22.413,	-57.649,	10.2,	1,	1,	1},
		{-20.663,	-74.382,	10.2,	1,	1,	1},
		{-18.897,	-91.002,	10.2,	1,	1,	1},
		{-17.198,	-107.875,	10.2,	1,	1,	1},
		{-15.551,	-124.441,	10.2,	1,	1,	1},
		{-13.828,	-141.021,	10.2,	1,	1,	1},
		{-12.112,	-157.898,	10.2,	1,	1,	1},
		{-10.355,	-174.451,	10.2,	1,	1,	1},
		{-8.553,	-191.245,	10.2,	1,	1,	1},
		
		-- Port runway line
		{-42.184,	-61.214,	10.2,	1,	1,	1},
		{-40.569,	-77.742,	10.2,	1,	1,	1},
		{-38.572,	-94.522,	10.2,	1,	1,	1},
		{-36.916,	-111.186,	10.2,	1,	1,	1},
		{-35.2825,	-127.906,	10.2,	1,	1,	1},
		{-33.649,	-144.626,	10.2,	1,	1,	1},
		{-31.973,	-161.421,	10.2,	1,	1,	1},
		{-30.304,	-177.919,	10.2,	1,	1,	1},
		{-28.421,	-194.810,	10.2,	1,	1,	1},
		
		-- Island safety lights
		{10.600,	-186.000,	29.8,	1,	0,	0},
		{10.600,	-153.800,	29.8,	1,	0,	0},
		{27.000,	-186.000,	29.8,	1,	0,	0},
		{27.000,	-153.800,	29.8,	1,	0,	0},

		-- Starboard navigation lights
		{7.100,		39.100,		10.2,	0,	1,	0},
		{24.500,	-20.000,	10.2,	0,	1,	0},
		{24.700,	-200.300,	10.2,	0,	1,	0},
		{24.6,		-110.15,	10.2,	0,	1,	0},

		-- Port navigation lights
		{-36.800,	-210.100,	10.2,	1,	0,	0},
		{-51.900,	-40.500,	10.2,	1,	0,	0},
		{-44.35,	-125.3,		10.2,	1,	0,	0},
		{-7.000,	39.000,		10.2,	1,	0,	0}
	}

----------------------
-- Reset everything --
----------------------
local function reset()
	ugfxCalled = 0
	
	-- Log event
	log("D", "CVN75_Script", "reset called")
end

----------------------------------------------
-- Spawn CVN-75 from Motorsports Playground --
----------------------------------------------
-- Call this function via the following:
-- scripts_HighBeam9000__MSPG__HeliCarrier_extension.spawnCVN75carrier()
local function cvn75_SpawnCarrier(carX, carY, carZ, carPhi)
	-- Get mission filename
	local missionFile = getMissionFilename()
	if missionFile == nil then
		log("E", "CVN75_Script", "No mission filename!")
		return
	end
	
	-- Split filename into path, file, ext
	local missionDir, missionName, missionExt = path.split(missionFile)
	if missionDir == nil then
		log("E", "CVN75_Script", "No mission directory!")
		return
	end
	
	-- Get map name
	local mapName = missionDir:sub(9, -2)
	if mapName == nil then
		log("E", "CVN75_Script", "No map name!")
		return
	end
	
	-- Spawn TSStatic
	local hcar = createObject('TSStatic')
	if hcar == nil then
		log("E", "CVN75_Script", "Cannot create TSStatic!")
		return
	end
	
	-- Set as helicarrier
	hcar:setField('shapeName', 0, "/levels/" .. mapName .. "/art/shapes/race/gerryfrod.dae")
	
	-- Set position, rotation and scale
	local quat = quatFromEuler(0, 0, carPhi / 180 * math.pi)
	hcar:setPosRot(carX, carY, carZ, quat.x, quat.y, quat.z, quat.w)
	hcar.scale = vec3(7, 7, 7)

	-- Set collision map stuff
	hcar:setField('collisionType', 0, "Visible Mesh Final")
	hcar:setField('decalType', 0, "Visible Mesh Final")

	-- Register helicarrier
	hcar.canSave = true
	hcar:registerObject('')
	
	-- One more collision thing
	be:reloadStaticCollision(false)
end

------------------------------
-- Calculate light position --
------------------------------
local function cvn75_CalcLightPos(carPos, lightPos, carPhi)
	
	-- Convert degrees to radians
	local convAng = -carPhi / 180 * math.pi
	
	-- Projected X coordinate
	local projX = carPos.x
		+ math.cos(convAng) * lightPos.x
		- math.sin(convAng) * lightPos.y
	
	-- Projected Y coordinate
	local projY = carPos.y
		+ math.cos(convAng) * lightPos.y
		+ math.sin(convAng) * lightPos.x
	
	-- Projected Z coordinate
	local projZ = carPos.z + lightPos.z
	
	-- Return transformed coordinates
	return vec3(projX, projY, projZ)
end

---------------------------------------------------------
-- Spawn lights for CVN-75 from Motorsports Playground --
---------------------------------------------------------
-- Call this function via the following:
-- scripts_HighBeam9000__MSPG__HeliCarrier_extension.spawnCVN75lights()
local function cvn75_SpawnCarrierLights(carX, carY, carZ, carPhi)
	
	-- Create point lights
	local lcnt = 0
	local plt = nil
	repeat
		
		-- Try to create this point light
		plt = createObject('PointLight')
		if plt == nil then
			log("E", "CVN75_Script", "Cannot create point light!")
			return
		end
		
		-- Set light color
		plt:setField('color', 0,
			tostring(lightOffsetsColors[lcnt + 1][4]) .. " " ..
			tostring(lightOffsetsColors[lcnt + 1][5]) .. " " ..
			tostring(lightOffsetsColors[lcnt + 1][6]) .. " " .. "1")
		
		-- Set light radius
		plt:setField('radius', 0, lightRadius)
		
		-- Set light brightness
		plt:setField('brightness', 0, lightBrightness)
		
		-- Set flare scale
		plt:setField('flareScale', 0, lightFlareScale)
		
		--Set flare type
		plt:setField('flareType', 0, lightFlareType)
		
		-- Set light position
		plt:setPosition(cvn75_CalcLightPos(
			vec3(carX, carY, carZ),
			vec3(lightOffsetsColors[lcnt + 1][1], lightOffsetsColors[lcnt + 1][2], lightOffsetsColors[lcnt + 1][3]),
			carPhi))
		
		-- Register light
		plt:registerObject('')
		
		-- Another one
		lcnt = lcnt + 1
		
	-- Keep going until all lights created or any light failed to create
	until(plt == nil or lcnt >= lightMaxCount)
end

------------------------------
-- Perform timed operations --
------------------------------
local function onUpdate(dtReal, dtSim, dtRaw)
	
	-- Log event
	if ugfxCalled == 0 then
		ugfxCalled = 1
		log("D", "CVN75_Script", "onUpdate called")
	end
end

----------------------
-- Public interface --
----------------------
M.spawnCVN75carrier = cvn75_SpawnCarrier
M.spawnCVN75lights = cvn75_SpawnCarrierLights
M.onExtensionLoaded = reset
M.onExtensionUnloaded = reset
M.reset = reset
M.onUpdate = onUpdate
return M
